package main

import "os"
import "fmt"
import "git.tebibyte.media/sashakoshka/fspl/cli"
import ferrors "git.tebibyte.media/sashakoshka/fspl/errors"

func main () {
	help := cli.NewFlag (
		'h', "help",
		"Display usage information and exit")
	format := cli.NewInputFlag (
		'm', "format",
		"Output format (.s, .o, .ll)", "",
		cli.NewValSet(".s", ".o", ".ll"))
	output := cli.NewInputFlag (
		'o', "output",
		"Output filename", "",
		cli.ValString)
	optimization := cli.NewInputFlag (
		'O', "optimization",
		"Optimization level (0-3)", "0",
		cli.NewValSet("0", "1", "2", "3"))

	application := cli.New (
		"Compile FSPL source files",
		help,
		format,
		output,
		optimization)
	
	application.Syntax = "[OPTION]... [FILE]..."
	application.ParseOrExit(os.Args)
	if help.Value != "" {
		application.Usage()
		os.Exit(2)
	}
	
	compiler := new(Compiler)
	compiler.Output       = output.Value
	compiler.Optimization = optimization.Value
	compiler.Format       = format.Value

	err := compiler.Compile(application.Args)
	if err != nil {
		fmt.Fprintf(os.Stderr, "%s: %v\n", os.Args[0], ferrors.Format(err))
		os.Exit(1)
	}
}
